@@ -8,7 +8,7 @@
 # from typing import *
 import time
 
-
+# TODO - bring EventTarget here and get rid of this one?
 class EventDispatcher(object):
     """ EventDispatcher is a class you can extend to give your obj event dispatching abilities """
 
@@ -297,14 +297,27 @@ class KeyboardEvent(Event):
     # location  Returns the location of a key on the keyboard or device KeyboardEvent
 
 
-class UiEvent(Event):
-    """ UiEvent """
+class UIEvent(Event):
+    """ UIEvent """
     def __init__(self, _type, *args, **kwargs):
         self.detail = None
         self.view = None
         super().__init__(_type, *args, **kwargs)
 
 
+class CompositionEvent(UIEvent):
+    """ CompositionEvent """
+
+    START = "compositionstart"
+    END = "compositionend"
+    UPDATE = "compositionupdate"
+
+    def __init__(self, _type, *args, **kwargs):
+        self.data = None  #: Returns the characters generated by the input method that raised the event
+        self.locale = None
+        super().__init__(_type, *args, **kwargs)
+
+
 class FocusEvent(Event):
     """ FocusEvent """
     BLUR = "blur"  #:
