@@ -24,6 +24,9 @@ from multiprocessing.pool import ThreadPool as Pool
 import re
 
 
+# true = True
+# false = False
+
 class Object(object):
     """ Object """
 
@@ -648,10 +651,7 @@ class Global(object):
 
     # def global
 
-    # def Infinity:
-    # pass
-    """A numeric value that represents positive/negative infinity """
-    # pass
+    Infinity = float("inf")
 
     @staticmethod
     def isFinite():
@@ -712,6 +712,18 @@ class Global(object):
         """ Deprecated in version 1.5. Use decodeURI() or decodeURIComponent() instead """
         # pass
 
+    @staticmethod
+    def require(path: str):
+        # if '.json' in path:
+            # TODO - loads json
+            # return
+
+        # '.'.join(path.split('/'))
+        # module = __import__(path)  # app.components.{component}
+        # my_class = getattr(module, component.title())
+        # return my_class()
+        pass
+
 
 class Date(Object):
     """ javascript date """
@@ -1339,6 +1351,11 @@ class Window(object):
 
 window = Window
 
+# class null():
+#     def __str__(self):
+#         return ''
+#     def __repr__(self):
+#         return None
 
 class Array(object):
     """ javascript array """
@@ -1351,6 +1368,11 @@ class Array(object):
             if isinstance(args[0], list):
                 self.args = args[0]
                 return
+            elif isinstance(args[0], int):
+                # self.args = [None] * args[0]
+                # self.args = [null()] * args[0]
+                self.args = [""] * args[0]
+                return
         self.args = list(args)
 
     def __getitem__(self, index):
@@ -1418,7 +1440,7 @@ class Array(object):
         except ValueError:
             return -1
         except Exception as e:
-            print(e)
+            # print(e)
             return -1
 
     def isArray(self):
@@ -1435,7 +1457,7 @@ class Array(object):
         try:
             return len(self.args) - self.args[::-1].index(value) - 1
         except Exception as e:
-            print(e)
+            # print(e)
             return None
 
     def pop(self):
@@ -1651,13 +1673,15 @@ class Number(float):
 
     # print(sys.float_info)
     MAX_VALUE = list(sys.float_info)[0]
-    MIN_VALUE = list(sys.float_info)[3]
-    # NEGATIVE_INFINITY Represents negative infinity (returned on overflow) Number
-    # POSITIVE_INFINITY Represents infinity (returned on overflow)  Number
+    MIN_VALUE = 5E-324  # CHANGE no longer >  list(sys.float_info)[3]
+
+    NEGATIVE_INFINITY = float("inf")  #: Represents negative infinity (returned on overflow) Number
+    POSITIVE_INFINITY = float("-inf")  #: Represents infinity (returned on overflow)  Number
+
     # prototype Allows you to add properties and methods to an object   Number
 
     def __init__(self, x="", *args, **kwargs):
-        self.x = x
+        self.x = Global.Number(x)
 
     def isInteger(self):
         """ Checks whether a value is an integer """
@@ -1667,18 +1691,98 @@ class Number(float):
         """ Checks whether a value is a safe integer """
         raise NotImplementedError
 
-    def toExponential(self, num):
+    def toExponential(self, num=None):
         """ Converts a number into an exponential notation """
-        return math.exp(num)
+        if num is not None:
+            exp = '{:e}'.format(Number(Number(self.x).toFixed(num)))
+        else:
+            exp = '{:e}'.format(self.x)
 
-    def toFixed(self, num):
-        """ Formats a number with x numbers of digits after the decimal point """
-        # return float(f"{0:.{{num}}f}".format(self.x))  # TODO - test
-        raise NotImplementedError
+        if 'e' in str(self.x):
+            exp = str(self.x)  # python already converts.
 
-    def toPrecision(self, num):
-        """ Formats a number to x length """
-        raise NotImplementedError
+        n = exp.split('e')[0].rstrip("0")
+        e = exp.split('e')[1].replace('00', '0')
+
+        if n == "0.":
+            n = "0"
+
+        if int(e) != 0:
+            if int(e) < 10 and int(e) > -10:  # TODO - not correct. lazy way to strip left 0s only
+                e = e.replace('0', '')
+
+        # print(  "AND:", n, "e" , e )
+        if n.endswith('.'):
+            n = n.strip('.')
+
+        return n + "e" + e
+
+    def toFixed(self, digits: int):
+        """[formats a number using fixed-point notation.]
+
+        Args:
+            digits ([int]): [The number of digits to appear after the decimal point
+
+        Returns:
+            [str]: [A string representing the given number using fixed-point notation.]
+        """
+        # print("DIGIT!", digits)
+        if digits < 0:
+            digits=0
+
+        fstring = "{:." + str(digits) + "f}"
+        return fstring.format(round(self.x, digits))
+
+    def toPrecision(self, precision):
+        """[returns a string representing the Number object to the specified precision.]
+
+        Args:
+            precision ([int]): [An integer specifying the number of significant digits.]
+
+        Returns:
+            [str]: [A string representing a Number object in fixed-point or exponential notation rounded to precision significant digits]
+        """
+        precision = int(precision)
+        # return str(math.pow(self.x, precision))
+        # raise NotImplementedError
+        return str(round(self.x, precision))
+
+    def toString(self, base: int):
+        """[returns a string representing the specified Number object.]
+
+        Args:
+            base (int): [An integer in the range 2 through 36 specifying the base to use for representing numeric values.]
+
+        Returns:
+            [str]: [a string representing the specified Number object]
+        """
+
+        if base is None:
+            return str(self.x)
+
+        import string
+        digs = string.digits + string.ascii_letters
+
+        if self.x < 0:
+            sign = -1
+        elif self.x == 0:
+            return digs[0]
+        else:
+            sign = 1
+
+        self.x *= sign
+        digits = []
+
+        while self.x:
+            digits.append(digs[int(self.x % base)])
+            self.x = int(self.x / base)
+
+        if sign < 0:
+            digits.append('-')
+
+        digits.reverse()
+
+        return ''.join(digits)
 
 
 class String(object):
@@ -1687,17 +1791,17 @@ class String(object):
     def __init__(self, x="", *args, **kwargs):
         # self.args = args
         # self.kwargs = kwargs
-        self.x = x
+        self.x = str(x)
 
     def __str__(self):
         return self.x
 
-    @staticmethod
+    # @staticmethod
     def charCodeAt(self, index: int):
         """ Returns the Unicode of the character at the specified index """
         return ord(self.x[index])
 
-    @staticmethod
+    # @staticmethod
     def fromCharCode(self, *codes):
         """ returns a string created from the specified sequence of UTF-16 code units """
         return "".join([str(chr(x)) for x in codes])
@@ -1721,6 +1825,8 @@ class String(object):
 
     def substring(self, start: int, end: int = None):
         """ Extracts the characters from a string, between two specified indices """
+        if start < 0:
+            start = 0
         if end is None:
             end = len(self.x)
         return self.x[start:end]
@@ -1744,7 +1850,7 @@ class String(object):
     def slice(self, start: int = 0, end: int = None):
         """ Selects a part of an string, and returns the new string """
         if end is None:
-            end = len(self.x) - 1
+            end = len(self.x)
         return self.x[start:end]
 
     def trim(self):
@@ -1752,12 +1858,15 @@ class String(object):
         return self.x.strip()
 
     def charAt(self, index: int):
-        """ Returns the character at the specified index (position) """
-        return self.x[index]
+        """[Returns the character at the specified index (position)]
 
-    # def test():
-        # r = (re.search(r"regexp", "someString") != None)
-        # return r
+        Args:
+            index (int): [index position]
+
+        Returns:
+            [str]: [character]
+        """
+        return self.x[index]
 
     def replace(self, old: str, new: str):
         """
@@ -1784,11 +1893,6 @@ class String(object):
     # """ Compares two strings in the current locale """
     # pass
 
-    # def search():
-    # """ Searches a string for a specified value, or regular expression, and returns the position of the match  """
-    # if re.search(r"\d", "iphone 8"):
-    # print("Has a number")
-
     def substr(self, start: int = 0, end: int = None):
         """ Extracts the characters from a string, beginning at a specified start position,
         and through the specified number of character """
@@ -1806,22 +1910,88 @@ class String(object):
         # locale.setlocale()
         return self.x.upper()
 
-    # def compile():
-    # """ Deprecated in version 1.5. Compiles a regular expression    RegExp """
-    # pass
-    # def lastIndex Specifies:
-    # """ the index at which to start the next match    RegExp """
-    # def test():
-    # """ Tests for a match in a string. Returns true or false    RegExp """
-    # pass
+    def indexOf(self, searchValue: str, fromIndex: int = 0):
+        """[returns the index within the calling String object of the first occurrence of the specified value, 
+        starting the search at fromIndex ]
 
-    # match()   Searches a string for a match against a regular expression, and returns the matches String
+        Args:
+            searchValue (str): [The string value to search for.]
+            fromIndex (int): [An integer representing the index at which to start the search]
+
+        Returns:
+            [type]: [The index of the first occurrence of searchValue, or -1 if not found.]
+
+        """
+        return self.x.index(searchValue, fromIndex)
+
+
+class RegExp():
+
+    def __init__(self, expression):
+        self.expression = expression
+        #self.flag  #: A string that contains the flags of the RegExp object.
+        #self.dotAll  #: Whether . matches newlines or not.
+        # self.global # Whether to test the regular expression against all possible matches in a string, or only against the first.
+        #self.hasIndices  # Whether the regular expression result exposes the start and end indices of captured substrings.
+        #self.ignoreCase  # Whether to ignore case while attempting a match in a string.
+        #self.multiline  # Whether or not to search in strings across multiple lines.
+        #self.source  # The text of the pattern.
+        #self.sticky  # Whether or not the search is sticky.
+        #self.unicode  # Whether or not Unicode features are enabled.
+        #self.lastIndex  # The index at which to start the next match.
+
+    def compile(self):
+        """ (Re-)compiles a regular expression during execution of a script. """ 
+        pass
+
+    def exec(self, s: str):
+        """ Executes a search for a match in its string parameter. """
+        # print("exec:", self.expression, s)
+        m = re.search(self.expression, s)
+        # print(m)
+        if (m):
+            return [s for s in m.groups()]
+
+    def test(self, s: str):
+        """[Tests for a match in its string parameter.]
+
+        Args:
+            s (str): [a string to match]
+
+        Returns:
+            [bool]: [True if match else False]
+        """
+        m = re.match(self.expression, s)
+        # print(m)
+        if (m):
+            return True
+        else:
+            return False
+
+    def toString(self):
+        """" Returns a string representing the specified object. Overrides the Object.prototype.toString() method. """
+        pass
+
+    def __str__(self):
+        """" Returns a string representing the specified object. Overrides the Object.prototype.toString() method. """
+        pass
+
+    # def [@@match]()
+    # Performs match to given string and returns match result.
+    # def [@@matchAll]()
+    # Returns all matches of the regular expression against a string.
+    # def [@@replace]()
+    # Replaces matches in given string with new substring.
+    # def [@@search]()
+    # Searches the match in given string and returns the index the pattern found in the string.
+    # def [@@split]()
+    # Splits given string into an array by separating the strin
 
 
 # https://developer.mozilla.org/en-US/docs/Web/API/URL
 
 class URL(object):
-    """ a tag extends from URL """
+    """ a-tag extends from URL """
 
     def __update__(self):
         # print( "update URL:", type(self), self  )
@@ -1830,6 +2000,7 @@ class URL(object):
             new = {}
             new['protocol'] = self.url.scheme
             new['hostname'] = self.url.hostname
+            new['href'] = self.url.geturl()
             new['port'] = self.url.port
             new['host'] = ''  # self.url.hostname
             new['pathname'] = self.url.path
@@ -1840,20 +2011,19 @@ class URL(object):
             new = {}
             new['protocol'] = self.protocol
             new['hostname'] = self.hostname
+            new['href'] = self.href
             new['port'] = self.port
             new['host'] = self.host
             new['pathname'] = self.pathname
             new['hash'] = self.hash  # self.hash
-
-            # rebuild
+            new['search'] = self.search  # self.url.query
+            new['_searchParams'] = self._searchParams  # URLSearchParams(self.url.query)
+            # NOTE - rebuild happening here
             self.url = urllib.parse.urlsplit(
-                new['protocol'] + "://" + new['host'] + new['pathname'] + new['hash'])
+                new['protocol'] + "://" + new['host'] + new['pathname'] + new['hash'] + new['search'])
 
-            new['search'] = self.url.query
-            new['_searchParams'] = URLSearchParams(self.url.query)
-
-            # reset
             self.href = self.url.geturl()
+
         except Exception:  # as e:
             # print('fails on props called by init as they dont exist yet')
             # print(e)
@@ -1868,16 +2038,13 @@ class URL(object):
             url (str): a url
         """
         self.url = urllib.parse.urlsplit(url)
-        self.href = self.url.geturl()
-
+        self.href = url  # self.url.geturl()
         self.protocol = self.url.scheme
         self.hostname = self.url.hostname
         self.port = self.url.port
         self.host = self.url.hostname
         self.pathname = self.url.path
         self.hash = ''
-
-        # print("HERE", self.url.query)
         self.search = self.url.query
         self._searchParams = URLSearchParams(self.url.query)
 
@@ -1892,6 +2059,7 @@ class URL(object):
 
     # @property
     # def href(self):
+    # TODO - check js vs tag. does js version remove query?. if so detect self.
     #     return self.href
 
     # @href.setter
@@ -1977,6 +2145,67 @@ class URL(object):
     # def origin(self):
         '''# origin    Returns the protocol, hostname and port number of a URL Location'''
 
+    def __str__(self):
+        return str(self.href)
+
+    # NOTE - node -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --
+    # @staticmethod
+    # def domainToASCII(domain: str):
+    #     """[It returns the Punycode ASCII serialization of the domain. 
+    #     If domain is an invalid domain, the empty string is returned.]
+
+    #     Args:
+    #         domain (str): [description]
+    #     """
+    #     pass
+
+    # @staticmethod
+    # def domainToUnicode(domain: str):
+    #     """[returns the Unicode serialization of the domain. 
+    #     If the domain is invalid, the empty string is returned]
+
+    #     Args:
+    #         domain (str): [description]
+    #     """
+    #     pass
+
+    # @staticmethod
+    # def fileURLToPath(url: str):
+    #     """[summary]
+
+    #     Args:
+    #         url (str): [description]
+    #     """
+    #     pass
+
+    # @staticmethod
+    # def format(URL, options):
+    #     """[summary]
+
+    #     Args:
+    #         URL ([type]): [description]
+    #         options ([type]): [description]
+    #     """
+    #     pass
+
+    # @staticmethod
+    # def pathToFileURL(path: str):
+    #     """[summary]
+
+    #     Args:
+    #         path (str): [description]
+    #     """
+    #     pass
+
+    # @staticmethod
+    # def urlToHttpOptions(url: str):
+    #     """[summary]
+
+    #     Args:
+    #         url (str): [description]
+    #     """
+    #     pass
+
 
 class URLSearchParams:
     """[utility methods to work with the query string of a URL]
