@@ -164,6 +164,10 @@ class Utils(object):
     def anagram(first, second):
         return Counter(first) == Counter(second)
 
+    @staticmethod
+    def is_palindrome(word):
+        return word == word[::-1]
+
     @staticmethod
     def acronym(sentence: str):
         """[pass a sentence, returns the acronym]
@@ -331,6 +335,17 @@ class Utils(object):
             print("No internet connection available.")
         return False
 
+    @staticmethod
+    def numberToBase(n, b):
+        if n == 0:
+            return [0]
+        digits = []
+        while n:
+            digits.append(int(n % b))
+            n //= b
+        return digits[::-1]
+
+
     # def any_iter(arr):
     #     ''' given a list. returns random until expired '''
     #     random.shuffle(arr)
